@@ -6,8 +6,9 @@ I/O device.
 __author__ = "Dennis van Gils"
 __authoremail__ = "vangils.dennis@gmail.com"
 __url__ = "https://github.com/Dennis-van-Gils/python-dvg-qdeviceio"
-__date__ = "04-07-2020"
-__version__ = "0.1.2"  # v0.0.1 on PyPI is based on prototype DvG_dev_Base__pyqt_lib.py v1.3.3
+__date__ = "15-07-2020"
+__version__ = "0.2.1"  # v0.0.1 on PyPI is based on prototype DvG_dev_Base__pyqt_lib.py v1.3.3
+# pylint: disable=protected-access
 
 from enum import IntEnum, unique
 import queue
@@ -55,7 +56,7 @@ def _cur_thread_name():
 
 
 @unique
-class DAQ_trigger(IntEnum):
+class DAQ_TRIGGER(IntEnum):
     """An enumeration decoding different modes of operation for
     :class:`Worker_DAQ` to perform a data-acquisition (DAQ) update.
     """
@@ -100,7 +101,7 @@ class QDeviceIO(QtCore.QObject):
         values. E.g., when making a `QDeviceIO` subclass specific to your
         Arduino project::
 
-            from dvg_qdeviceio import QDeviceIO, DAQ_trigger
+            from dvg_qdeviceio import QDeviceIO, DAQ_TRIGGER
 
             class Arduino_qdev(QDeviceIO):
                 def __init__(
@@ -111,7 +112,7 @@ class QDeviceIO(QtCore.QObject):
 
                     # Set the DAQ to 10 Hz internal timer
                     self.create_worker_DAQ(
-                        DAQ_trigger                = DAQ_trigger.INTERNAL_TIMER,
+                        DAQ_trigger                = DAQ_TRIGGER.INTERNAL_TIMER,
                         DAQ_function               = DAQ_function,
                         DAQ_interval_ms            = 100,  # 100 ms -> 10 Hz
                         critical_not_alive_count   = 3,
@@ -156,6 +157,7 @@ class QDeviceIO(QtCore.QObject):
             :class:`PyQt5.QtCore.QObject`.
 
     .. _`QDeviceIO_attributes`:
+    .. rubric:: Attributes:
 
     Attributes:
         dev (:obj:`object` | :obj:`None`):
@@ -422,7 +424,7 @@ class QDeviceIO(QtCore.QObject):
         """
         if self._thread_DAQ is None:
             pft(
-                "Worker_DAQ %s: Can't start thread, because it does not exist. "
+                "Worker_DAQ  %s: Can't start thread, because it does not exist. "
                 "Did you forget to call 'create_worker_DAQ()' first?"
                 % self.dev.name
             )
@@ -430,7 +432,7 @@ class QDeviceIO(QtCore.QObject):
 
         elif not self.dev.is_alive:
             dprint(
-                "\nWorker_DAQ %s: WARNING - Device is not alive.\n"
+                "\nWorker_DAQ  %s: WARNING - Device is not alive.\n"
                 % self.dev.name,
                 ANSI.RED,
             )
@@ -452,7 +454,7 @@ class QDeviceIO(QtCore.QObject):
         self._qwc_worker_DAQ_started.wait(self._mutex_wait_worker_DAQ)
         locker_wait.unlock()
 
-        if self.worker_DAQ._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        if self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             # Wait a tiny amount of extra time for the worker to have entered
             # 'self._qwc.wait(self._mutex_wait)' of method '_do_work()'.
             # Unfortunately, we can't use
@@ -465,7 +467,7 @@ class QDeviceIO(QtCore.QObject):
             # signal event.
             time.sleep(0.05)
 
-        if self.worker_DAQ._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        if self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             # We expect a 'signal_DAQ_paused' being emitted at start-up by this
             # worker. Make sure this signal gets processed as soon as possible,
             # and prior to any other subsequent actions the user might request
@@ -550,6 +552,7 @@ class QDeviceIO(QtCore.QObject):
         Returns:
             True if successful, False otherwise.
         """
+
         if self._thread_DAQ is None or self.worker_DAQ._started_okay is None:
             # CASE: quit() without create()
             #   _thread_DAQ == None
@@ -567,18 +570,27 @@ class QDeviceIO(QtCore.QObject):
         #   _thread_DAQ == QThread()
         #   _started_okay = True
 
+        if self._thread_DAQ.isFinished():
+            # CASE: Device has had a 'connection_lost' event during run-time,
+            # which already stopped and closed the thread.
+            print(
+                "Closing thread %s already closed."
+                % "{:.<16}".format(self._thread_DAQ.objectName())
+            )
+            return True
+
         if self.worker_DAQ.debug:
             tprint(
                 "Worker_DAQ  %s: stop requested..." % self.dev.name, ANSI.WHITE,
             )
 
-        if self.worker_DAQ._DAQ_trigger == DAQ_trigger.INTERNAL_TIMER:
+        if self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
             # The QTimer inside the INTERNAL_TIMER '_do_work()'-routine /must/
             # be stopped from within the worker_DAQ thread. Hence, we must use a
             # signal from out of this different thread.
             self._signal_stop_worker_DAQ.emit()
 
-        elif self.worker_DAQ._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        elif self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             # The QWaitCondition inside the SINGLE_SHOT_WAKE_UP '_do_work()'-
             # routine will likely have locked worker_DAQ. Hence, a
             # '_signal_stop_worker_DAQ' signal as above might not get handled by
@@ -587,7 +599,7 @@ class QDeviceIO(QtCore.QObject):
             # which is perfectly fine for SINGLE_SHOT_WAKE_UP as per my design.
             self.worker_DAQ._stop()
 
-        elif self.worker_DAQ._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        elif self.worker_DAQ._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             # We directly call '_stop()' from out of this different thread,
             # which is perfectly fine for CONTINUOUS as per my design.
             self.worker_DAQ._stop()
@@ -616,6 +628,7 @@ class QDeviceIO(QtCore.QObject):
         Returns:
             True if successful, False otherwise.
         """
+
         if self._thread_jobs is None or self.worker_jobs._started_okay is None:
             # CASE: quit() without create()
             #   _thread_DAQ == None
@@ -633,6 +646,15 @@ class QDeviceIO(QtCore.QObject):
         #   _thread_DAQ == QThread()
         #   _started_okay = True
 
+        if self._thread_jobs.isFinished():
+            # CASE: Device has had a 'connection_lost' event during run-time,
+            # which already stopped the worker and closed the thread.
+            print(
+                "Closing thread %s already closed."
+                % "{:.<16}".format(self._thread_jobs.objectName())
+            )
+            return True
+
         if self.worker_jobs.debug:
             tprint(
                 "Worker_jobs %s: stop requested..." % self.dev.name, ANSI.WHITE,
@@ -670,7 +692,7 @@ class QDeviceIO(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def pause_DAQ(self):
-        """Only useful in mode :const:`DAQ_trigger.CONTINUOUS`. Request
+        """Only useful in mode :const:`DAQ_TRIGGER.CONTINUOUS`. Request
         :attr:`worker_DAQ` to pause and stop listening for data. After
         :attr:`worker_DAQ` has achieved the `paused` state, it will emit
         :obj:`signal_DAQ_paused()`.
@@ -680,7 +702,7 @@ class QDeviceIO(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def unpause_DAQ(self):
-        """Only useful in mode :const:`DAQ_trigger.CONTINUOUS`. Request
+        """Only useful in mode :const:`DAQ_TRIGGER.CONTINUOUS`. Request
         :attr:`worker_DAQ` to resume listening for data. Once
         :attr:`worker_DAQ` has successfully resumed, it will emit
         :obj:`signal_DAQ_updated()` for every DAQ update.
@@ -690,7 +712,7 @@ class QDeviceIO(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def wake_up_DAQ(self):
-        """Only useful in mode :const:`DAQ_trigger.SINGLE_SHOT_WAKE_UP`.
+        """Only useful in mode :const:`DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP`.
         Request :attr:`worker_DAQ` to wake up and perform a single update,
         i.e. run :attr:`~Worker_DAQ.DAQ_function` once. It will emit
         :obj:`signal_DAQ_updated()` after :attr:`~Worker_DAQ.DAQ_function` has
@@ -797,9 +819,9 @@ class Worker_DAQ(QtCore.QObject):
             .. _`arg_DAQ_trigger`:
 
         DAQ_trigger (:obj:`int`, optional):
-            Mode of operation. See :class:`DAQ_trigger`.
+            Mode of operation. See :class:`DAQ_TRIGGER`.
 
-            Default: :const:`DAQ_trigger.INTERNAL_TIMER`.
+            Default: :const:`DAQ_TRIGGER.INTERNAL_TIMER`.
 
             .. _`arg_DAQ_function`:
 
@@ -821,7 +843,8 @@ class Worker_DAQ(QtCore.QObject):
                 worker. (You're basically undermining the reason to have
                 multithreading in the first place). That could be acceptable,
                 though, when you need to print debug or critical error
-                information to the terminal, but be aware about this warning.
+                information to the terminal, but be aware about the possible
+                negative effects.
 
                 Instead, connect to :meth:`QDeviceIO.signal_DAQ_updated` from
                 out of the *main/GUI* thread to instigate changes to the
@@ -854,13 +877,13 @@ class Worker_DAQ(QtCore.QObject):
                         return True         # Return success
 
         DAQ_interval_ms (:obj:`int`, optional):
-            Only useful in mode :const:`DAQ_trigger.INTERNAL_TIMER`. Desired
+            Only useful in mode :const:`DAQ_TRIGGER.INTERNAL_TIMER`. Desired
             data-acquisition update interval in milliseconds.
 
             Default: :const:`100`.
 
         DAQ_timer_type (:obj:`PyQt5.QtCore.Qt.TimerType`, optional):
-            Only useful in mode :const:`DAQ_trigger.INTERNAL_TIMER`.
+            Only useful in mode :const:`DAQ_TRIGGER.INTERNAL_TIMER`.
             The update interval is timed to a :class:`PyQt5.QtCore.QTimer`
             running inside :class:`Worker_DAQ`. The default value
             :const:`PyQt5.QtCore.Qt.PreciseTimer` tries to ensure the best
@@ -885,7 +908,7 @@ class Worker_DAQ(QtCore.QObject):
             .. _`arg_calc_DAQ_rate_every_N_iter`:
 
         calc_DAQ_rate_every_N_iter (:obj:`int`, optional):
-            The increase the accuracy of calculating the DAQ rate, it is advised
+            To increase the accuracy of calculating the DAQ rate, it is advised
             to average over several DAQ updates, i.e. iterations. It will take
             at least *N* updates before :attr:`QDeviceIO.obtained_DAQ_rate_Hz`
             will contain the calculated rate.
@@ -902,6 +925,8 @@ class Worker_DAQ(QtCore.QObject):
             All remaining keyword arguments will be passed onto inherited class
             :class:`PyQt5.QtCore.QObject`.
 
+    .. rubric:: Attributes:
+
     Attributes:
         qdev (:class:`QDeviceIO`):
             Reference to the parent :class:`QDeviceIO` class instance.
@@ -928,7 +953,7 @@ class Worker_DAQ(QtCore.QObject):
     def __init__(
         self,
         qdev,
-        DAQ_trigger=DAQ_trigger.INTERNAL_TIMER,
+        DAQ_trigger=DAQ_TRIGGER.INTERNAL_TIMER,
         DAQ_function=None,
         DAQ_interval_ms=100,
         DAQ_timer_type=QtCore.Qt.PreciseTimer,
@@ -954,7 +979,7 @@ class Worker_DAQ(QtCore.QObject):
         self._started_okay = None
 
         # Members specifically for INTERNAL_TIMER
-        if self._DAQ_trigger == DAQ_trigger.INTERNAL_TIMER:
+        if self._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
             self._timer = QtCore.QTimer()
             self._timer.setInterval(DAQ_interval_ms)
             self._timer.setTimerType(DAQ_timer_type)
@@ -966,7 +991,7 @@ class Worker_DAQ(QtCore.QObject):
             #        round(1e3/DAQ_interval_ms), 1)
 
         # Members specifically for SINGLE_SHOT_WAKE_UP
-        elif self._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             self._running = True
             self._qwc = QtCore.QWaitCondition()
             self._mutex_wait = QtCore.QMutex()
@@ -974,7 +999,7 @@ class Worker_DAQ(QtCore.QObject):
         # Members specifically for CONTINUOUS
         # Note: At start-up, the worker will directly go into a paused state
         # and trigger a 'signal_DAQ_paused' PyQt signal
-        elif self._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        elif self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             self._running = True
             self._pause = None  # Will be set at init of '_do_work()' when 'start_worker_DAQ()' is called
             self._paused = None  # Will be set at init of '_do_work()' when 'start_worker_DAQ()' is called
@@ -1020,12 +1045,12 @@ class Worker_DAQ(QtCore.QObject):
             )
 
         # INTERNAL_TIMER
-        if self._DAQ_trigger == DAQ_trigger.INTERNAL_TIMER:
+        if self._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
             self._timer.start()
             confirm_has_started(self)
 
         # SINGLE_SHOT_WAKE_UP
-        elif self._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             locker_wait = QtCore.QMutexLocker(self._mutex_wait)
             locker_wait.unlock()
 
@@ -1067,11 +1092,12 @@ class Worker_DAQ(QtCore.QObject):
             # Wait a tiny amount for the other thread to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100, lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll()
+                100,
+                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
         # CONTINUOUS
-        elif self._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        elif self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             while self._running:
                 if init:
                     self._pause = True
@@ -1124,7 +1150,8 @@ class Worker_DAQ(QtCore.QObject):
             # in a different thread than this one, to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100, lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll()
+                100,
+                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
     @_coverage_resolve_trace
@@ -1211,7 +1238,7 @@ class Worker_DAQ(QtCore.QObject):
         if self.debug:
             tprint("Worker_DAQ  %s: stopping" % self.dev.name, self.debug_color)
 
-        if self._DAQ_trigger == DAQ_trigger.INTERNAL_TIMER:
+        if self._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
             # NOTE: The timer /must/ be stopped from the worker_DAQ thread!
             self._timer.stop()
 
@@ -1224,14 +1251,15 @@ class Worker_DAQ(QtCore.QObject):
             # Wait a tiny amount for the other thread to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100, lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll()
+                100,
+                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
             )
 
-        elif self._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             self._running = False
             self._qwc.wakeAll()  # Wake up for the final time
 
-        elif self._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        elif self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             self._running = False
 
     # ----------------------------------------------------------------------
@@ -1240,12 +1268,12 @@ class Worker_DAQ(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def pause(self):
-        """Only useful in mode :const:`DAQ_trigger.CONTINUOUS`. See the
+        """Only useful in mode :const:`DAQ_TRIGGER.CONTINUOUS`. See the
         description at :meth:`QDeviceIO.pause_DAQ`.
 
         This method can be called from another thread.
         """
-        if self._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        if self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             if self.debug:
                 tprint(
                     "Worker_DAQ  %s: pause requested..." % self.dev.name,
@@ -1259,12 +1287,12 @@ class Worker_DAQ(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def unpause(self):
-        """Only useful in mode :const:`DAQ_trigger.CONTINUOUS`. See the
+        """Only useful in mode :const:`DAQ_TRIGGER.CONTINUOUS`. See the
         description at :meth:`QDeviceIO.unpause_DAQ`.
 
         This method can be called from another thread.
         """
-        if self._DAQ_trigger == DAQ_trigger.CONTINUOUS:
+        if self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
             if self.debug:
                 tprint(
                     "Worker_DAQ  %s: unpause requested..." % self.dev.name,
@@ -1282,12 +1310,12 @@ class Worker_DAQ(QtCore.QObject):
 
     @QtCore.pyqtSlot()
     def wake_up(self):
-        """Only useful in mode :const:`DAQ_trigger.SINGLE_SHOT_WAKE_UP`. See the
+        """Only useful in mode :const:`DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP`. See the
         description at :meth:`QDeviceIO.wake_up_DAQ`.
 
         This method can be called from another thread.
         """
-        if self._DAQ_trigger == DAQ_trigger.SINGLE_SHOT_WAKE_UP:
+        if self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             if self.debug:
                 tprint(
                     "Worker_DAQ  %s: wake requested..." % self.dev.name,
@@ -1398,6 +1426,8 @@ class Worker_jobs(QtCore.QObject):
             All remaining keyword arguments will be passed onto inherited class
             :class:`PyQt5.QtCore.QObject`.
 
+    .. rubric:: Attributes:
+
     Attributes:
         qdev (:class:`QDeviceIO`):
             Reference to the parent :class:`QDeviceIO` class instance.
@@ -1511,7 +1541,8 @@ class Worker_jobs(QtCore.QObject):
         # Wait a tiny amount for the other thread to have entered the
         # QWaitCondition lock, before giving a wakingAll().
         QtCore.QTimer.singleShot(
-            100, lambda: self.qdev._qwc_worker_jobs_stopped.wakeAll()
+            100,
+            lambda: self.qdev._qwc_worker_jobs_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
         )
 
     @_coverage_resolve_trace
@@ -1530,12 +1561,11 @@ class Worker_jobs(QtCore.QObject):
                 self.debug_color,
             )
 
-        """Process all jobs until the queue is empty. We must iterate 2
-        times because we use a sentinel in a FIFO queue. First iter
-        removes the old sentinel. Second iter processes the remaining
-        queue items and will put back a new sentinel again.
-        """
-        for i in range(2):
+        # Process all jobs until the queue is empty. We must iterate 2 times
+        # because we use a sentinel in a FIFO queue. First iter removes the old
+        # sentinel. Second iter processes the remaining queue items and will put
+        # back a new sentinel again.
+        for _i in range(2):
             for job in iter(self._queue.get_nowait, self._sentinel):
                 func = job[0]
                 args = job[1:]
@@ -1559,7 +1589,7 @@ class Worker_jobs(QtCore.QObject):
                     # Send I/O operation to the device
                     try:
                         func(*args)
-                    except Exception as err:
+                    except Exception as err:  # pylint: disable=broad-except
                         pft(err)
                 else:
                     # User-supplied job processing
